@@ -57,6 +57,11 @@ BASELINES_RESOURCE = Aliases(
     "baseline",
     "baselines",
 )
+DEPLOYMENTS_RESOURCE = Aliases(
+    ["deployments", "deployment", "deploys", "deploy"],
+    "deployment",
+    "deployments",
+)
 NAMESPACES_RESOURCE = Aliases(
     ["namespaces", "name", "names", "namesp", "namesps", "namespace"],
     "namespace",
@@ -139,16 +144,17 @@ CONFIG_ALIAS = Aliases(
 DEL_RESOURCES: List[str] = [POLICIES_RESOURCE.name]
 GET_RESOURCES: List[str] = [
     CLUSTERS_RESOURCE.name_plural,
+    CONNECTIONS_RESOURCE.name_plural,
+    DEPLOYMENTS_RESOURCE.name_plural,
     FINGERPRINTS_RESOURCE.name_plural,
     MACHINES_RESOURCE.name_plural,
     NAMESPACES_RESOURCE.name_plural,
     NODES_RESOURCE.name_plural,
+    OPSFLAGS_RESOURCE.name_plural,
     PODS_RESOURCE.name_plural,
     POLICIES_RESOURCE.name_plural,
-    REDFLAGS_RESOURCE.name_plural,
-    OPSFLAGS_RESOURCE.name_plural,
     PROCESSES_RESOURCE.name_plural,
-    CONNECTIONS_RESOURCE.name_plural,
+    REDFLAGS_RESOURCE.name_plural,
 ]
 VAL_RESOURCES: List[str] = [
     BASELINES_RESOURCE.name,
@@ -820,6 +826,7 @@ class ArgumentParametersCommand(CustomCommand):
         super().__init__(*args, **kwargs)
         self.base_param_count = len(self.params)
         self.argument_value = self.argument_name
+        self.unspecific = True
 
     def parse_args(self, ctx, args: List[str]) -> List[str]:
         args_cpy = args.copy()
@@ -835,6 +842,7 @@ class ArgumentParametersCommand(CustomCommand):
                 break
         argument_value = ctx.params.get(self.argument_name)
         if argument_value:
+            self.unspecific = False
             for obj in self.argument_value_parameters:
                 for value_option in obj[self.argument_name]:
                     if argument_value == value_option:
@@ -849,6 +857,15 @@ class ArgumentParametersCommand(CustomCommand):
         """Writes all the options into the formatter if they exist."""
         opts = []
         specif_opts = []
+        specific_index = {}
+        if self.unspecific:
+            for obj in self.argument_value_parameters:
+                index = ", ".join(
+                    str(option) for option in obj[self.argument_name]
+                )
+                specific_index[index] = len(obj["args"])
+                for arg_maker in obj["args"]:
+                    arg_maker(self)
         for i, param in enumerate(self.get_params(ctx)):
             rv = param.get_help_record(ctx)
             if rv is not None:
@@ -861,8 +878,17 @@ class ArgumentParametersCommand(CustomCommand):
             with formatter.section("Options"):
                 formatter.write_dl(opts)
         if specif_opts:
-            with formatter.section(f"Options for {self.argument_value}"):
-                formatter.write_dl(specif_opts)
+            if self.unspecific:
+                index = 0
+                for options, num in specific_index.items():
+                    with formatter.section(f"Options for {options}"):
+                        formatter.write_dl(
+                            specif_opts[index : index + num]  # noqa E203
+                        )
+                    index = index + num
+            else:
+                with formatter.section(f"Options for {self.argument_value}"):
+                    formatter.write_dl(specif_opts)
 
 
 class MutuallyExclusiveOption(click.Option):
@@ -1307,6 +1333,31 @@ FILE_EXT_MAP = {
 }
 
 
+def find_resource_filename(data: Dict, default: str = "spyctl_output") -> str:
+    """Checks metadata for a suitable filename, if not it
+    resorts to the kind field lowercased and spaces removed.
+    If all else fails, uses the default.
+
+    Args:
+        data (Dict): The resource that will be outputted
+        default (str): Default string to use for filename if all else fails.
+
+    Returns:
+        str: A valid filename for use in saving data to disk.
+    """
+    rv = data.get(METADATA_FIELD, {}).get(METADATA_NAME_FIELD)
+    if rv:
+        rv = slugify(rv)
+        rv.replace(" ", "_")
+    if not rv:
+        rv: str = data.get(KIND_FIELD)
+        if rv:
+            rv = rv.strip(" ").replace(" ", "_").lower()
+    if not rv:
+        rv = default
+    return rv
+
+
 def unique_fn(fn: str, output_format) -> Optional[str]:
     count = 1
     file_ext = FILE_EXT_MAP[output_format]
